#include <QApplication>
#include <QDesktopWidget>
#include <QPainter>
#include <QMouseEvent>
#include <QMoveEvent>
#include <QRect>
#include <QPoint>
#include "grabber.h"

Grabber::Grabber(QWidget &grabbedwin, QString &title)
  : QWidget(), m_grabbed(false), m_moved(false)
{
  const QRect as = QApplication::desktop()->availableGeometry(this);
  QPoint p(as.left() + 50, as.top() + 50);

  m_grab = &grabbedwin;
  m_title = title;

  setWindowFlags(Qt::Window
		 |Qt::WindowStaysOnTopHint
		 |Qt::FramelessWindowHint
		 |Qt::X11BypassWindowManagerHint);

  move(p);
  resize(100, 20);
  setContentsMargins(0, 0, 0, 0);
}

void
Grabber::closeEvent(QCloseEvent *e)
{
  e->ignore();
}

void
Grabber::mousePressEvent(QMouseEvent *e)
{
  if (e->buttons() & Qt::LeftButton)
    {
      m_grabpt = e->pos();
      grabMouse();
    }
}

void
Grabber::mouseReleaseEvent(QMouseEvent *e)
{
  if (Qt::LeftButton == e->button())
    {
      if (!m_moved)
	m_grab->setVisible(!m_grab->isVisible());

      releaseMouse();
      m_moved = false;
    }
}

void
Grabber::mouseMoveEvent(QMouseEvent *e)
{
  if (e->buttons() & Qt::LeftButton) {
    const QRect as = QApplication::desktop()->availableGeometry(this);
    QPoint newp;
    
    m_moved = true;

    if (e->globalX() - m_grabpt.x() < as.left())
      newp.setX(as.left());
    else if (e->globalX() - m_grabpt.x() + m_grab->width() > as.right())
      newp.setX(as.right() - m_grab->width());
    else
      newp.setX(e->globalX() - m_grabpt.x());

    if (e->globalY() - m_grabpt.y() < as.top())
      newp.setY(as.top());
    else if (e->globalY() - m_grabpt.y()
	     + m_grab->height() + height() > as.bottom())
      newp.setY(as.bottom() - m_grab->height() - height());
    else
      newp.setY(e->globalY() - m_grabpt.y());

    move(newp);
  }
}

void
Grabber::moveEvent(QMoveEvent *e)
{
  m_grab->move(e->pos().x(), e->pos().y() + height());
}

void
Grabber::paintEvent(QPaintEvent *)
{
  QPainter p(this);

  p.setBackground(QColor(0x00, 0x55, 0x99));
  p.eraseRect(rect());
  p.setPen(Qt::white);
  p.drawRect(0, 0, width() - 1, height() - 1);
  p.drawText(rect(), Qt::AlignCenter, m_title);
}
