#include <QLabel>
#include <QTime>
#include "infowin.h"

InfoWin::InfoWin()
  : QWidget(), m_stime(-1), m_updateTimeTimerId(-1)
{
  setupUi(this);
  setWindowFlags(Qt::Window
		 |Qt::WindowStaysOnTopHint
		 |Qt::FramelessWindowHint
		 |Qt::X11BypassWindowManagerHint);
  setContentsMargins(1, 1, 1, 1);

  activateWindow();
}

void
InfoWin::timerEvent(QTimerEvent *timer)
{
  if (timer->timerId() == m_updateTimeTimerId)
    updateTime();
}

void
InfoWin::enablePassword(bool enable)
{
  passwordButton->setEnabled(enable);
}

void
InfoWin::setStartTime(int ntime)
{
  m_stime = ntime;
}

void
InfoWin::setOwed(unsigned int amount)
{
  owedLabel->setText(QString::number(amount / 100.0, 'f', 2));
}

void
InfoWin::setOther(unsigned int amount)
{
  otherLabel->setText(QString::number(amount / 100.0, 'f', 2));
}

void
InfoWin::updateTime()
{
  if (0 < m_stime)
    {
      QTime ttime(0, 0, 0);
    
      timeLabel->setText(ttime.addSecs(time(NULL)
			 - m_stime).toString("hh:mm:ss"));
    }
  else
    timeLabel->setText("--:--:--");
}

void
InfoWin::connectionStatus(bool works)
{
  if (!works)
    {
      owedLabel->setText("-.--");
      otherLabel->setText("-.--");
    }
}

void
InfoWin::enableTimeUpdate(bool enable)
{
  if (enable)
    m_updateTimeTimerId = startTimer(1000);
  else
    killTimer(m_updateTimeTimerId);
}

void
InfoWin::on_passwordButton_clicked()
{
  emit passwordEdit();
}

void
InfoWin::on_exitButton_clicked()
{
  emit endSession();
}
