#include <QApplication>
#include <QDesktopWidget>
#include <QDir>
#include <QRegExp>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QPainter>
#include <QPen>
#include <QRadialGradient>
#if defined(Q_WS_MAC)
# include <Carbon/Carbon.h>
#elif defined (Q_WS_WIN)
# if defined(UNICODE)
#   undef UNICODE
# endif
# include <windows.h>
# include <winuser.h>
# define UNICODE
#endif
#include "lockwin.h"

#if defined(Q_WS_WIN32)
#include "qcclc.h"

static bool isScreenLocked = false;
static unsigned int ps;
static HHOOK kHook = NULL;
static HHOOK mHook = NULL;

static LRESULT CALLBACK mylpfn(int nCode, WPARAM wp, LPARAM lp)
{
  KBDLLHOOKSTRUCT *kbh = (KBDLLHOOKSTRUCT *)lp;
  BOOL ctrlDown = FALSE;

  if (isScreenLocked)
    {
      if (nCode == HC_ACTION)
	{
	  ctrlDown =
	    GetAsyncKeyState(VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1);
	  if (kbh->vkCode == VK_TAB && kbh->flags & LLKHF_ALTDOWN)
	    return 1; // Alt+Tab
	  if (kbh->vkCode == VK_ESCAPE && ctrlDown)
	    return 1; // Ctrl+Escape
	  if (kbh->vkCode == VK_ESCAPE && kbh->flags & LLKHF_ALTDOWN)
	    return 1; // Alt+Escape
	  if (kbh->vkCode == VK_LWIN || kbh->vkCode == VK_RWIN)
	    return 1; // Win
	  // otherwise
	  return CallNextHookEx(kHook, nCode, wp, lp);
	} 
    } 
    
  return CallNextHookEx(kHook, nCode, wp, lp);
}
#endif

LockWin::LockWin()
  : QWidget(), m_input(), m_memberlogin(), m_memberid(-1),
    m_box(0, 0, 400, 100), m_lockpix()
{
  setWindowFlags(Qt::Window
		 |Qt::WindowStaysOnTopHint
		 |Qt::FramelessWindowHint
		 |Qt::X11BypassWindowManagerHint);

  m_box.moveTo(QApplication::desktop()->width() / 2 - m_box.width() / 2,
	     QApplication::desktop()->height() / 2 - m_box.height() / 2);

  if (QDir::current().exists("lockpix"))
    m_lockpix.load("lockpix");

  m_color.bg.setRgb(0, 0, 0);
  m_color.authBoxBg.setRgb(0, 0, 0);
  m_color.authBoxText.setRgb(255, 255, 255);
  m_color.authBoxTextShadow.setRgb(0, 0, 0);
  m_color.authBoxTitleBg.setRgb(100, 100, 140);
  m_color.authBoxTitleText.setRgb(255, 255, 255);
  m_color.authBoxTitleTextShadow.setRgb(0, 0, 0);
  m_color.authBoxBorder.setRgb(255, 255, 255);
}

void
LockWin::setBgColor(const QString &color)
{
  m_color.bg.setNamedColor(color);
}

void
LockWin::setAuthBoxBgColor(const QString &color)
{
  m_color.authBoxBg.setNamedColor(color);
}

void
LockWin::setAuthBoxTextColor(const QString &color)
{
  m_color.authBoxText.setNamedColor(color);
}

void
LockWin::setAuthBoxTextShadowColor(const QString &color)
{
  m_color.authBoxTextShadow.setNamedColor(color);
}

void
LockWin::setAuthBoxTitleBgColor(const QString &color)
{
  m_color.authBoxTitleBg.setNamedColor(color);
}

void
LockWin::setAuthBoxTitleTextColor(const QString &color)
{
  m_color.authBoxTitleText.setNamedColor(color);
}

void
LockWin::setAuthBoxTitleTextShadowColor(const QString &color)
{
  m_color.authBoxTitleTextShadow.setNamedColor(color);
}

void
LockWin::setAuthBoxBorderColor(const QString &color)
{
  m_color.authBoxBorder.setNamedColor(color);
}

void
LockWin::mousePressEvent(QMouseEvent *e)
{
  if (e->y() <= 40 && m_allowUserLogin)
    emit userStart();
}

void
LockWin::keyPressEvent(QKeyEvent *e)
{
  if (!m_allowMemberLogin) return;
  
  if (e->key() >= Qt::Key_Space && e->key() <= Qt::Key_ydiaeresis)
    {
      //QRegExp numeric("^\\d*$");

      //if (m_memberid != -1 || numeric.exactMatch(e->text()))
      m_input.append(e->text());

      m_drawAuthBox = true;
      update(m_box);
    }
  else if (e->key() == Qt::Key_Escape)
    {
      m_drawAuthBox = false;
      m_input.clear();
      m_memberlogin.clear();
      m_memberid = -1;
      update(m_box);
    }
  else if (e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete)
    {
      m_input.truncate(m_input.length() - 1);
      update(m_box);
    }
  else if (e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return)
    {
      if (-1 == m_memberid)
	{
	  m_drawAuthBox = true;
	  if (!m_input.isEmpty())
	    {
	      m_memberid = m_input.toInt();
	      m_memberlogin = m_input;
	      m_input.clear();
	    }
	  update(m_box);
	}
      else
	{
	  m_drawAuthBox = false;
	  if (m_memberid)
	    emit memberStart(m_memberid, m_input);
	  else
	    emit memberStart(m_memberlogin, m_input);
	  m_input.clear();
	  m_memberlogin.clear();
	  m_memberid = -1;
	  update(m_box);
	}
    }
  else
    e->ignore();
}
  
void
LockWin::paintEvent(QPaintEvent *e)
{
  QRect sg = QApplication::desktop()->screenGeometry(this);
  QPainter p(this);
  QFont font(p.font());

  p.setClipRect(e->rect());
  p.setBackground(m_color.bg);
  p.eraseRect(sg);
  if (m_allowUserLogin)
    p.fillRect(0, 0, sg.width(), 40, QColor(0x44, 0x44, 0x55));

  if (m_lockpix.isNull())
    {
      int logosize = sg.height() / 2;
      QRect logo(sg.width() / 2 - logosize / 2,
		 sg.height() / 2 - logosize / 2,
		 logosize,
		 logosize);
      QRadialGradient gradient(logo.x() + logo.width() / 2,
			       logo.y() + logo.height() / 2,
			       logo.width() / 2,
			       logo.x() + logo.width() / 4,
			       logo.y() + logo.height() / 4);

      gradient.setColorAt(0.0, Qt::white);
      gradient.setColorAt(0.8, QColor(0x00, 0x55, 0x55));
      gradient.setColorAt(1.0, Qt::black);
      p.setBrush(gradient);
      p.setPen(QPen(Qt::white, 10.0));
      p.drawEllipse(logo);
      font.setBold(true);
      font.setPixelSize(int(logosize / 2.2));
      p.setFont(font);
      p.drawText(logo, Qt::AlignCenter, "CCL");
    }
  else
    p.drawImage(sg.width() / 2 - m_lockpix.width() / 2,
		sg.height() / 2 - m_lockpix.height() / 2,
		m_lockpix);

  font.setPixelSize(30);
  p.setPen(Qt::black);
  p.setFont(font);
  if (m_allowUserLogin)
    p.drawText(0, 0, sg.width(), 40, Qt::AlignCenter,
	       tr("Click here to begin"));
  p.setPen(QPen(Qt::white, 10.0));
  if (m_allowUserLogin)
    p.drawText(-2, -2, sg.width(), 40, Qt::AlignCenter,
	       tr("Click here to begin"));

  if (m_drawAuthBox)
    drawAuthBox(p);
}

void
LockWin::drawAuthBox(QPainter &p)
{
  QString title = (-1 == m_memberid) ? tr("Member ID:") : tr("Password:");
  QString text = m_input + "|";
  QFont font(p.font());

  font.setBold(true);
  font.setPixelSize(18);

  if (-1 != m_memberid)
    text.replace(0, text.length() - 1, QString(text.length() - 1, '*'));

  m_box.moveTo(QApplication::desktop()->width() / 2 - m_box.width() / 2,
	       QApplication::desktop()->height() / 2 - m_box.height() / 2);

  p.setBrush(Qt::NoBrush);
  p.setPen(QPen(m_color.authBoxBorder, 10.0));
  p.fillRect(m_box, m_color.authBoxBg);
  p.fillRect(m_box.x(), m_box.y(), m_box.width(), 30, m_color.authBoxTitleBg);
  p.drawRect(m_box);
  
  p.setFont(font);
  p.setPen(m_color.authBoxTitleTextShadow);
  p.drawText(m_box.x() + 2, m_box.y() + 2, m_box.width(), 30, Qt::AlignCenter,
	     title);
  p.setPen(m_color.authBoxTitleText);
  p.drawText(m_box.x(), m_box.y(), m_box.width(), 30, Qt::AlignCenter, title);
  p.setPen(m_color.authBoxTextShadow);
  p.drawText(m_box.x() + 2, m_box.y() + 2, m_box.width(), m_box.height(),
	     Qt::AlignCenter, text); 
  p.setPen(m_color.authBoxText);
  p.drawText(m_box, Qt::AlignCenter, text); 
}

void
LockWin::lock()
{
  setGeometry(QApplication::desktop()->screenGeometry());
#if defined(Q_WS_MAC)
  show();
#else
  showFullScreen();
#endif
  raise();
  activateWindow();
  grabKeyboard();
#if defined(Q_WS_MAC)
  SetSystemUIMode(kUIModeAllHidden,
		  kUIOptionDisableAppleMenu
		  |kUIOptionDisableProcessSwitch
		  |kUIOptionDisableForceQuit
		  |kUIOptionDisableSessionTerminate);
#elif defined(Q_WS_WIN)
  OSVERSIONINFO osversion;

  osversion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osversion);

  if (osversion.dwPlatformId == VER_PLATFORM_WIN32_NT) // NT/2000/XP
    {
      HWND hwnd = FindWindow("Shell_traywnd", NULL);

      EnableWindow(hwnd, FALSE);
      // Disable the task manager
      HKEY hk;
      DWORD val = 1;
      const char *key =
	"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\System";
      
      if (RegOpenKey(HKEY_CURRENT_USER, key, &hk) != ERROR_SUCCESS)
	RegCreateKey(HKEY_CURRENT_USER, key, &hk);
      RegSetValueEx(hk, "DisableTaskMgr", 0, REG_DWORD,(BYTE *)&val,
		    sizeof(val));
      // Add Hooks
      if (!kHook)
	kHook = SetWindowsHookEx(WH_KEYBOARD_LL, (HOOKPROC)mylpfn,
				 (HINSTANCE)GetModuleHandle(NULL), 0);
      /*  if (!mHook)
       *   mHook = SetWindowsHookEx (WH_MOUSE,(HOOKPROC)mylpfn,
       *    (HINSTANCE)GetModuleHandle(NULL),0);
       */
    }
  else // All the above was only for NT/2000/XP, this is for Win 9x/Me
    SystemParametersInfo(SPI_SETSCREENSAVERRUNNING, TRUE, &ps, 0);

  isScreenLocked = true;
#endif
}

void
LockWin::unlock()
{
  m_input.clear();
  m_memberlogin.clear();
  m_memberid = -1;
  update();
  hide();
  releaseKeyboard();
#if defined(Q_WS_MAC)
  SetSystemUIMode(kUIModeNormal, 0);
#elif defined(Q_WS_WIN)
  OSVERSIONINFO osversion;

  osversion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osversion);

  if (osversion.dwPlatformId == VER_PLATFORM_WIN32_NT) // NT/2000/XP
    {
      // Show the taskbar
      HWND hwnd = FindWindow("Shell_traywnd", NULL);

      EnableWindow(hwnd, TRUE);
      // Reenable the task manager
      HKEY hk;
      const char *key =
	"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\System";
      if (RegOpenKey(HKEY_CURRENT_USER, key, &hk) != ERROR_SUCCESS)
	RegCreateKey(HKEY_CURRENT_USER, key, &hk);
      RegDeleteValue(hk, "DisableTaskMgr");
      // Remove Hooks
      if (kHook)
	{
	  UnhookWindowsHookEx(kHook);
	  kHook = NULL;
	}
      if (mHook)
	{
	  UnhookWindowsHookEx(mHook);
	  mHook = NULL;
	}
    }
  else // 9x/Me
    SystemParametersInfo(SPI_SETSCREENSAVERRUNNING, FALSE, &ps, 0);

  isScreenLocked = false;
#endif
}

void
LockWin::allowMemberLogin(bool allow)
{
  m_allowMemberLogin = allow;
  if (!m_allowMemberLogin)
    {
      m_memberid = -1;
      m_memberlogin.clear();
      m_input.clear();
      repaint(m_box);
    }
}

void
LockWin::allowUserLogin(bool allow)
{
  m_allowUserLogin = allow;
  repaint(0, 0, QApplication::desktop()->screenGeometry(this).width(),40);
}
