#ifndef QCCLC_H
#define QCCLC_H
#include <QObject>
#include <QApplication>
#include <QTranslator>
#include <time.h>

class Grabber;
class PasswordDialog;
class InfoWin;
class LockWin;

class Qcclc : public QApplication
{
  Q_OBJECT
public:
  Qcclc(int &argc, char **argv);
  virtual ~Qcclc();
  void loadSettings();
  void loadLockWinSettings();
  void dispatchCommand(unsigned int cmd, void *data, unsigned int size);
  void shutdownNetworking();
  bool initNetworking();
  bool isScreenLocked();
  void showMessage(const char *message);
protected:
  virtual void timerEvent(QTimerEvent *timer);
private:
  void initCCLC();
  bool initCCLC(const char *cafile, const char *certfile,
		const char *certpass);
  void shutdownCCLC();
  bool parseCommandLine();
  void printHelp();
public slots:
  void start();
  void stop();
  void resume();
  void userStart();
  void memberStart(int id, QString password);
  void memberStart(QString login, QString password);
  void passwordEdit();
  void setPassword(const QString &oldpass, const QString &newpass);
  void endSession();
  void lockScreen();
  void unlockScreen();
  void turnScreenOff();
  void shutdownSystem();
  void rebootSystem();
  void updateStatus();
  void checkEvents();
  void tryConnect();
signals:
  void timeChanged(int time);
  void owedChanged(unsigned int owed);
  void otherChanged(unsigned int owed);
  void statusChanged(bool active);
  void timeoutChanged(int timeout);
  void connectionStatusChanged(bool works);
  void enablePassword(bool enable);
  void allowMemberLogin(bool enable);
  void allowUserLogin(bool enable);
private:
  bool			 m_initialized;
  bool			 m_networking;
  bool			 m_active;
  time_t		 m_stime;
  time_t		 m_timeout;
  QTranslator		 m_translator;
  struct {
    QString server, myname;
    unsigned short port;
  } m_connectionParams;
  struct {
    QString lang;
    QString grabberTitle;
  } m_config;

  Grabber		*m_grabber;
  PasswordDialog	*m_passwordDialog;
  InfoWin		*m_infoWin;
  LockWin		*m_lockWin;
  int			 m_updateStatusTimerId;
  int			 m_checkEventsTimerId;
  int			 m_tryConnectTimerId;
};

#endif
